package distraction;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import processing.core.*;
import rita.*;

// Is going "in" for 0,0 even if 0,0 is not out yet. 
// Sometimes removes the background alert, but doesn't remove the text. 
// Sometimes doesn't remove the background alert. 

// Toy with times to make it 5:00, eventually fill up the screen. 

// OPTIONAL: Make the popup the only thing on the screen, before the rest of the alerts start. (Not too much time... though...) 

public class Distraction11 extends PApplet {

	private static final long serialVersionUID = -3822618250671142677L;

	// Fonts
	PFont courier, helvetica, helveticaBold16, helveticaBold24, lucidaGrandeBold;

	// Text entry
	RiText entry;
	String entryContent, word, searchTerm, actualSearchTerm;
	int lineCount, charCount;
	boolean cursorVisible;
	boolean shiftPressed;

	// Livejournal interface
	RiText postAnEntry, date, dateField, subject, subjectField;
	// PImage logo;
	PImage logo2010;

	// Clouds
	ArrayList<ArrayList<RiText>> titlesToDisplay; 

	// Growl interface
	PImage[][] alerts = new PImage[4][6];
	boolean[][] isAlertVisible = new boolean[4][6];
	// createWords(this, processing.core.PFont pfont, java.lang.String[] lines, float startX, float startY) 
	PImage search;
	PImage search2;
	PImage autosearch;
	// PImage autosearch2;
	boolean isSearchVisible, isSearch2Visible, isAutosearchVisible;
	int searchX, searchY, autosearchCount;
	
	int jOut, kOut;

	// Lull timer
	int lullStart;
	boolean canCallDisplay, isTyping, cannotErase;
	double attentionSpan = 1000; // the time it takes to get distracted, and inverse time it takes to erase it

	/**
	 * Setup
	 */
	public void setup() {
		size(1280,1024);

		courier = loadFont("Courier-16.vlw");
		helvetica = loadFont("Helvetica-16.vlw");
		helveticaBold16 = loadFont("Helvetica-Bold-16.vlw");
		helveticaBold24 = loadFont("Helvetica-Bold-24.vlw");
		lucidaGrandeBold = loadFont("LucidaGrande-Bold-12.vlw");

		// Livejournal interface
		RiText.setDefaultColor(0);
		postAnEntry = new RiText(this, "Post an Entry", 25, 145);
		postAnEntry.textFont(helveticaBold24);
		postAnEntry.setColor(0, 48, 96);

		date = new RiText(this, "Date:", 25, 180);
		date.textFont(helveticaBold16);

		dateField = new RiText(this, "December 15, 2010, " + hour() + ":" + minute(), 130, 180);
		dateField.textFont(helvetica);

		subject = new RiText(this, "Subject:", 25, 210);
		subject.textFont(helveticaBold16);

		subjectField = new RiText(this, "Distraction", 130, 210);
		subjectField.textFont(helvetica);

		// logo = loadImage("lj.png");
		logo2010 = loadImage("lj2010.png");

		// Text entry

		entry = new RiText(this, 35, 250);
		entry.textFont(courier);

		entryContent = "";
		word = "";
		searchTerm = "";
		actualSearchTerm = "";
		shiftPressed = false;

		// Clouds backend
		titlesToDisplay = new ArrayList<ArrayList<RiText>>();
		canCallDisplay = false;
		isTyping = false;
		cannotErase = false;

		// Alerts backend

		// Load images and set alerts not visible
		PImage alert = loadImage("growl.png");

		for (int i = 0; i < 4; i++) {
			for (int j = 0; j < 6; j++) {
				alerts[i][j] = alert;
				isAlertVisible[i][j] = false;
			}
		}

		search = loadImage("search.png");
		search2 = loadImage("search2.png");
		isSearchVisible = false;
		isSearch2Visible = false;
		autosearch = loadImage("autosearch.png");
		// autosearch2 = loadImage("autosearch2.png");
		isAutosearchVisible = false;
		// isAutosearch2Visible = false;
		autosearchCount = 3;
		
		jOut = -1;
		kOut = -1;

		// Timer
		new RiTimer(this, .5f);
		cursorVisible = false;
	}

	/**
	 * times the cursor, and decreases attention span
	 * @param re
	 */
	public void onRiTaEvent(RiTaEvent re) { 

		if (cursorVisible == false) {
			entry.setText(entryContent + "|");
			cursorVisible = true;
		} else {
			entry.setText(entryContent);
			cursorVisible = false;
		}

		if (attentionSpan >= 100 && !isTyping && autosearchCount == 0) {
			attentionSpan = 1000;		// test
			// attentionSpan = attentionSpan - .75;		
			// 	attentionSpan = attentionSpan - 1;		// the real one
		}

	}

	/**
	 * Text entry that also calls erase
	 */
	public void keyPressed() {

		if (key != '\n' && key != ' ' && key != '\b') {

			// Reset lull timer
			lullStart = millis();
			canCallDisplay = true;
			isTyping = true;

			// Here, call the method (a thread) to erase things if it's the first time the key has been pressed 
			// since displaying. 
			if (cannotErase == false) {
				Thread r = new Thread() {
					public void run() {
						cannotErase = true;
						erase();
					}
				};
				r.start();

			}
		}

		// Text entry

		if ((key>='A' && key <= 'z') || (key>='0' && key <= '9') || key == ' ' || key == '-' || key == ';' ||
				key == '=' || key == '[' || key == ']' || key == ',' || key == '.' || key == '/' || key == '\n' || 
				key == '\'') {

			// ? !
			if (shiftPressed == true && (key == '/' || key == '1' || key == ';' || key == '\"')) {

				actualSearchTerm = searchTerm;
				searchTerm = "";

				Thread t = new Thread() {
					public void run() {
						println("[SEARCH] " + actualSearchTerm);
						search(actualSearchTerm);
						actualSearchTerm = "";
					}
				};
				t.start();

				if (key == '/') {
					entryContent += "?";
					word += "?";
				} else if (key == '1') {
					entryContent += "!";
					word += "!";
				} else if (key == ';') {
					entryContent += ":";
					word += ":";
				} 
			} else if (shiftPressed == true && (key == '9' || key == '0' || key == '\'')) {
				if (key == '\'') {
					entryContent += "\"";
					word += "\"";
				} else if (key == '9') {
					entryContent += "(";
					word += "(";
				} else if (key == '0') {
					entryContent += ")";
					word += ")";
				}
			}
			else if (key == '\n') {
				entryContent += key;
			} else {
				entryContent += key; // add key to content
				searchTerm += key;
			}

			shiftPressed = false;

			entry.setText(entryContent);
			charCount++;

			if (key == ' ') {
				word = "";
			} else {
				word += key;
			}

			// backspacing
		} else if (key == '\b') {

			try {
				entryContent = entryContent.substring(0, entryContent.length() - 1);
				entry.setText(entryContent);
				charCount--;

				// if it goes back a line
				if (entryContent.endsWith("\n")) {
					charCount = 93;
					lineCount--;
				}
			} catch (StringIndexOutOfBoundsException e1) {
				return;
			}

			try {
				word = word.substring(0, word.length() - 1);
			} catch (StringIndexOutOfBoundsException e1) {
				word = "";
			}

			try {
				searchTerm = searchTerm.substring(0, searchTerm.length() - 1);
			} catch (StringIndexOutOfBoundsException e1) {
				searchTerm = "";
			}

		} else if (keyCode == SHIFT) {
			shiftPressed = true;
		}

		// new line by pressing \n
		if (key == '\n') {

			if (lineCount > 19) {
				entryContent = entryContent.substring(entryContent.indexOf("\n") + 1, entryContent.length());
				entry.setText(entryContent);
			}

			charCount = 0;
			lineCount++;
		}

		// new line the natural way
		if (charCount == 95) { 

			if (lineCount > 19) {
				entryContent = entryContent.substring(entryContent.indexOf("\n") + 1, entryContent.length());
				entry.setText(entryContent);
			}

			if (word.length() < 95) {
				entryContent = entryContent.substring(0, entryContent.length() - word.length());
				entryContent += '\n' + word;
			}
			else {
				entryContent += '\n';
			}

			charCount = 0;
			lineCount++;
		}

		// Delimiting, getting search terms
		if (searchTerm != "" && (key == '.' || key == ',' || key == '-' || key == ';')) {	//  || word == "and"
			searchTerm = searchTerm.substring(0, searchTerm.length() - 1);

			actualSearchTerm = searchTerm;
			searchTerm = "";

			Thread t = new Thread() {
				public void run() {
					println("[SEARCH] " + actualSearchTerm);
					search(actualSearchTerm);
				}
			};
			t.start();
		}
	}

	public void mousePressed() {

		if (mouseButton == RIGHT && autosearchCount > 0) {
			searchX = mouseX;
			searchY = mouseY;
			isSearchVisible = true;
		} else if (mouseButton == LEFT) {
			if (isAutosearchVisible) {
				if (mouseX >= 746 && mouseX <= 835 && mouseY >= 202 && mouseY <= 223) {

					/*	Thread t = new Thread() {
						public void run() {
							canCallDisplay = false;
							cannotErase = false;
							display(titlesToDisplay.get(titlesToDisplay.size() - 1));
						}
					};
					t.start();	*/
					isAutosearchVisible = false;
				}
			}

			if (isSearch2Visible || isSearchVisible) {
				if (mouseX >= searchX && mouseX <= searchX + 174 && mouseY >= searchY + 5 && mouseY <= searchY + 23) {

					autosearchCount--;
					isSearch2Visible = false;

					if (autosearchCount == 0) isAutosearchVisible = true;
					else {

						Thread t = new Thread() {
							public void run() {
								canCallDisplay = false;
								cannotErase = false;
								display(titlesToDisplay.get(titlesToDisplay.size() - 1));
							}
						};
						t.start();
					}

				} else {
					isSearchVisible = false;
					isSearch2Visible = false;
				}
			}
		}
	}

	public void mouseMoved() {
		if (isSearchVisible || isSearch2Visible) {
			if (mouseX >= searchX && mouseX <= searchX + 174 && mouseY >= searchY + 5 && mouseY <= searchY + 23) {
				isSearch2Visible = true;
				isSearchVisible = false;
			} else {
				isSearchVisible = true;
				isSearch2Visible = false;
			}
		}
	}

	/** 
	 * Search the term in Google and create an array of RiText
	 * @param term
	 */
	public void search(String term) {

		RiHtmlParser fetcher = new RiHtmlParser(this);
		term = term.replace(' ', '+'); 	// convert the term's spaces to pluses

		String url = "http://www.google.com/search?q=" + term;

		String[] titleArray = fetcher.fetchLinkText(url);
		String[] linkArray = fetcher.fetchLinks(url);

		ArrayList<RiText> titleArrayList = new ArrayList<RiText>();

		for (int i = 11; i < titleArray.length - 5; i++) {
			if (!titleArray[i].contains("Cached") && 
					!titleArray[i].contains("Similar pages") &&
					!titleArray[i].contains("related articles") &&
					!titleArray[i].contains("Image results for") &&
					!titleArray[i].contains("News results for") &&
					!titleArray[i].contains("prose%25200") &&
					(titleArray[i].length() > 2)) {
				// println(titleArray[i]);

				// make a new RiText, set not visible.

				String link = "";
				RiText thisTitle;

				// println(titleArray[i]);

				link = extractLink(linkArray[i+1]);
				if (link != "") link = link.substring(7);
				// println(link);

				// TODO: Break sentence up by spaces
				// TEST: Cut off, to see what is actually overlapping. 
				if (titleArray[i].length() > 31) {
					titleArray[i] = titleArray[i].substring(0, 31) + "\n" + titleArray[i].substring(31, titleArray[i].length());
				}

				if (link.length() > 31) {
					if (link.length() > 62) link = link.substring(0, 31) + "\n" + link.substring(31, 62);
					else link = link.substring(0, 31) + "\n" + link.substring(31, link.length());
				}

				titleArray[i] = titleArray[i] + "\n\n" + link;

				thisTitle = new RiText(this, titleArray[i]);
				thisTitle.textFont(lucidaGrandeBold);
				thisTitle.setColor(255);
				thisTitle.setVisible(false);

				// add it to the array list.
				if (link != "") titleArrayList.add(thisTitle);
			}
		}

		titlesToDisplay.add(titleArrayList);

	}

	/**
	 * Extracts link from <a href=""> code
	 * @param content the <a href=""> code
	 * @return the link
	 */
	public String extractLink(String content) {

		String linkRegEx = "http(s)?://[^&]*";

		Pattern pattern = Pattern.compile(linkRegEx);
		Matcher matcher = pattern.matcher(content);

		// Find all the matches.
		String match = "";

		while (matcher.find()) {
			match = matcher.group();
			// println(match);
		}
		return match;
	}

	/**
	 * Display RiText
	 */
	public void display(ArrayList<RiText> titleArrayList) {

		int start = 0;
		int i = 0;
		boolean canDisplay = true;
		int displayTimer = 10; // times the 'leftover' Google results once I've typed something

		// Start displaying them one by one
		while (i < (titleArrayList.size() + 9)) {

			if (millis() - start > attentionSpan){			

				if (isTyping == true) {
					canDisplay = false;
					displayTimer = (int) (10 - (attentionSpan / 1000) * 10);
					// println(displayTimer);
				}

				start = millis();

				int j = -1;
				int k = -1;

				if (i < titleArrayList.size() && (displayTimer > 0)) {

					// set the position of the RiText (go through while loop)
					j = 0;
					k = 0;

					boolean found = false;

					// println("0,0: " + isAlertVisible[0][0]);

					while (j < 4 && !found) {
						k = 0;
						while (k < 6 && !found) {
							if (!isAlertVisible[j][k] && (j != jOut && k != kOut)) found = true;
							// isn't working currently
							else k++;
						}
						if (!found) j++;
					}

					if (!found) {
						println("not found");
						return;
					}

					println("in: " + j + ", " + k);

					isAlertVisible[j][k] = true;
					titleArrayList.get(i).setPosition(1028 - 312*j, 34 + 112*k);
					titleArrayList.get(i).setVisible(true);

					if (!canDisplay)
						displayTimer--;
				}

				// Erase, naturally
				if (i >= 9) {

				//	jOut = -1;
				//	kOut = -1;

					jOut = (int) ((1028 - titleArrayList.get(i - 9).getX()) / 312);
					kOut = (int) ((titleArrayList.get(i - 9).getY() - 34) / 112);

					// if j and k = jOut and kOut, don't let it!
					if (jOut == j && kOut == k || !isAlertVisible[jOut][kOut]) println("didn't set on display");
					else { 
						isAlertVisible[jOut][kOut] = false;
						titleArrayList.get(i - 9).setVisible(false);
						println("out on display: " + jOut + ", " + kOut);
					}
				}
				i++;
			}
		}
	}

	/**
	 * Erase RiText
	 */
	public void erase() {

		if (!titlesToDisplay.isEmpty()) {

			int eraseTimer = millis();

			for (int i = titlesToDisplay.size() - 1; i >= 0; i--) {
				for (int j = titlesToDisplay.get(i).size() - 1; j >= 0; j--) {
					if (titlesToDisplay.get(i).get(j).isVisible()) {
						eraseTimer = millis();

						while (millis() - eraseTimer < 1000) {	// test
							// while (millis() - eraseTimer < (1250 - attentionSpan)) {

						}

						// titlesToDisplay.get(i).get(j).fadeOut(1f);

					//	jOut = 0;
					//	kOut = 0;

						jOut = (int) ((1028 - titlesToDisplay.get(i).get(j).getX()) / 312);
						kOut = (int) ((titlesToDisplay.get(i).get(j).getY() - 34) / 112);

						if (isAlertVisible[jOut][kOut]) {
							titlesToDisplay.get(i).get(j).setVisible(false);
							isAlertVisible[jOut][kOut] = false;
							println("out on erase: " + jOut + ", " + kOut);
						} else println("didn't set on erase");

					}
				}
			}
		}
	}

	/**
	 * Draws the layout and continually checks for lulls in my typing, then displaying
	 */
	public void draw() {

		background(255);

		// Livejournal logo
		image(logo2010,0,0);

		// subject field
		stroke(0);
		fill(255);
		rect(125, 191, 900, 23);

		// entry box
		rect(25, 225, 1000, 500);

		// search menu
		tint(255,255);
		if (isSearchVisible) image(search,searchX,searchY);
		if (isSearch2Visible) image(search2,searchX,searchY);

		// autosearch dialog
		if (isAutosearchVisible) image(autosearch,433,87);
		// if (isAutosearch2Visible) image(autosearch2,433,87);

		// Growl images (test)
		tint(255,240);		// fade from 0-240 to fade in
		for (int i = 0; i < 4; i++) {
			for (int j = 0; j < 6; j++) {
				// image(alerts[i][j], 968 - 312*i, 12 + 112*j);
				if(isAlertVisible[i][j]) image(alerts[i][j], 968 - 312*i, 12 + 112*j);
			}
		}

		// Continually check for lull in typing, then displays

		if (millis() - lullStart > 200)
			isTyping = false;

		if (millis() - lullStart > attentionSpan && !titlesToDisplay.isEmpty()) {
			// println("display called, but can't call display");
			if (canCallDisplay && autosearchCount == 0) {

				// isTyping = false;

				Thread t = new Thread() {
					public void run() {
						canCallDisplay = false;
						cannotErase = false;
						display(titlesToDisplay.get(titlesToDisplay.size() - 1));
					}
				};
				t.start();

			}
		}
	}


	public static void main(String args[])
	{
		PApplet.main(new String[] { distraction.Distraction11.class.getName() });
	}

}