package distraction;

import java.util.*;

import processing.core.*;
import rita.*;

// Correlate typing to focusing - and search in the lulls between typing

public class Distraction8 extends PApplet {

	private static final long serialVersionUID = -3822618250671142677L;
	
	// Text entry
	RiText entry;
	String entryContent, word, searchTerm, actualSearchTerm;
	int lineCount, charCount;
	boolean cursorVisible;
	boolean shiftPressed;

	// Livejournal interface
	RiText postAnEntry;
	RiText subject;
	RiTextField subjectField;
	PImage logo;

	// Clouds
	ArrayList<ArrayList<RiText>> titlesToDisplay; 
	float[][] colors = new float[4][4];

	// Lull timer
	int lullStart;
	boolean canCallDisplay;
	boolean isTyping;
	boolean cannotErase;
	double attentionSpan = 1000; // the time it takes to get distracted, and inverse time it takes to erase it

	/**
	 * Setup
	 */
	@SuppressWarnings("deprecation")
	public void setup() {
		size(1280,1024);
		// frameRate(120);
		
		// fonts
		// PFont helvetica30 = 
		// look this up

		// Livejournal interface
		RiText.createDefaultFont(this, "helvetica", 30);
		RiText.setDefaultColor(0, 48, 96);
		postAnEntry = new RiText(this, "Post an Entry", 25, 150);

		RiText.createDefaultFont(this, "helvetica", 16);
		RiText.setDefaultColor(0);

		subject = new RiText(this, "Subject:", 25, 200);
		subjectField = new RiTextField(this, 90, 183, 935, 20, 18, "Distraction");

		logo = loadImage("lj.png");

		// Text entry
		RiText.createDefaultFont(this, "courier", 16);
		entry = new RiText(this, 35, 250);
		entryContent = "";
		word = "";
		searchTerm = "";
		actualSearchTerm = "";
		shiftPressed = false;

		// Clouds backend
		titlesToDisplay = new ArrayList<ArrayList<RiText>>();
		canCallDisplay = false;
		isTyping = false;
		cannotErase = false;

		// blue
		colors[0][0] = 75;
		colors[0][1] = 120;
		colors[0][2] = 232;
		colors[0][3] = 200;
		// red
		colors[1][0] = 213;
		colors[1][1] = 19;
		colors[1][2] = 46;
		colors[1][3] = 200;
		// yellow
		colors[2][0] = 239;
		colors[2][1] = 146;
		colors[2][2] = 3;
		colors[2][3] = 200;
		// green
		colors[3][0] = 8;
		colors[3][1] = 167;
		colors[3][2] = 24;
		colors[3][3] = 200;

		// Timer
		new RiTimer(this, .5f);
		cursorVisible = false;
	}

	/**
	 * times the cursor, and decreases attention span
	 * @param re
	 */
	public void onRiTaEvent(RiTaEvent re) { 

		if (cursorVisible == false) {
			entry.setText(entryContent + "|");
			cursorVisible = true;
		} else {
			entry.setText(entryContent);
			cursorVisible = false;
		}

		if (attentionSpan >= 100 && !isTyping) {
			attentionSpan = attentionSpan - .75;		// .5 when linear (isTyping is not a condition)
			// println(attentionSpan);
		}

	}

	/**
	 * Text entry that also calls erase
	 */
	public void keyPressed() {

		// Reset lull timer
		lullStart = millis();
		canCallDisplay = true;
		isTyping = true;

		// Here, call the method (a thread) to erase things if it's the first time the key has been pressed 
		// since displaying. 
		if (cannotErase == false) {
			Thread r = new Thread() {
				public void run() {
					cannotErase = true;
					erase();
				}
			};
			r.start();

		}

		// Text entry

		if ((key>='A' && key <= 'z') || (key>='0' && key <= '9') || key == ' ' || key == '-' || key == ';' ||
				key == '=' || key == '[' || key == ']' || key == ',' || key == '.' || key == '/' || key == '\n' || 
				key == '\'') {

			// ? !
			if (shiftPressed == true && (key == '/' || key == '1' || key == ';' || key == '\"')) {

				actualSearchTerm = searchTerm;
				searchTerm = "";

				Thread t = new Thread() {
					public void run() {
						println("[SEARCH] " + actualSearchTerm);
						search(actualSearchTerm);
						actualSearchTerm = "";
					}
				};
				t.start();

				if (key == '/') {
					entryContent += "?";
					word += "?";
				} else if (key == '1') {
					entryContent += "!";
					word += "!";
				} else if (key == ';') {
					entryContent += ":";
					word += ":";
				} 
			} else if (shiftPressed == true && (key == '9' || key == '0' || key == '\'')) {
				if (key == '\'') {
					entryContent += "\"";
					word += "\"";
				} else if (key == '9') {
					entryContent += "(";
					word += "(";
				} else if (key == '0') {
					entryContent += ")";
					word += ")";
				}
			}
			else if (key == '\n') {
				entryContent += key;
			} else {
				entryContent += key; // add key to content
				searchTerm += key;
			}

			shiftPressed = false;

			entry.setText(entryContent);
			charCount++;

			if (key == ' ') {
				word = "";
			} else {
				word += key;
			}

			// backspacing
		} else if (key == '\b') {

			try {
				entryContent = entryContent.substring(0, entryContent.length() - 1);
				entry.setText(entryContent);
				charCount--;

				// if it goes back a line
				if (entryContent.endsWith("\n")) {
					charCount = 93;
					lineCount--;
				}
			} catch (StringIndexOutOfBoundsException e1) {
				return;
			}

			try {
				word = word.substring(0, word.length() - 1);
			} catch (StringIndexOutOfBoundsException e1) {
				word = "";
			}

			try {
				searchTerm = searchTerm.substring(0, searchTerm.length() - 1);
			} catch (StringIndexOutOfBoundsException e1) {
				searchTerm = "";
			}

		} else if (keyCode == SHIFT) {
			shiftPressed = true;
		}

		// new line by pressing \n
		if (key == '\n') {

			if (lineCount > 19) {
				entryContent = entryContent.substring(entryContent.indexOf("\n") + 1, entryContent.length());
				entry.setText(entryContent);
			}

			charCount = 0;
			lineCount++;
		}

		// new line the natural way
		if (charCount == 95) { 

			if (lineCount > 19) {
				entryContent = entryContent.substring(entryContent.indexOf("\n") + 1, entryContent.length());
				entry.setText(entryContent);
			}

			if (word.length() < 95) {
				entryContent = entryContent.substring(0, entryContent.length() - word.length());
				entryContent += '\n' + word;
			}
			else {
				entryContent += '\n';
			}

			charCount = 0;
			lineCount++;
		}

		// Delimiting, getting search terms
		if (searchTerm != "" && (key == '.' || key == ',' || key == '-' || key == ';')) {	//  || word == "and"
			searchTerm = searchTerm.substring(0, searchTerm.length() - 1);

			actualSearchTerm = searchTerm;
			searchTerm = "";

			Thread t = new Thread() {
				public void run() {
					println("[SEARCH] " + actualSearchTerm);
					search(actualSearchTerm);
				}
			};
			t.start();
		}
	}

	/** 
	 * Search the term in Google and create an array of RiText
	 * @param term
	 */
	@SuppressWarnings("deprecation")
	public void search(String term) {

		int color = 4;

		RiHtmlParser fetcher = new RiHtmlParser(this);
		term = term.replace(' ', '+'); 	// convert the term's spaces to pluses

		String url = "http://www.google.com/search?q=" + term;

		String[] titleArray = fetcher.fetchLinkText(url);

		ArrayList<RiText> titleArrayList = new ArrayList<RiText>();

		for (int i = 11; i < titleArray.length - 5; i++) {
			if (!titleArray[i].contains("Cached") && 
					!titleArray[i].contains("Similar pages") &&
					!titleArray[i].contains("related articles") &&
					!titleArray[i].contains("Image results for") &&
					!titleArray[i].contains("News results for") &&
					(titleArray[i].length() > 2)) {
				// println(titleArray[i]);

				// make a new RiText, set not visible.

				RiText.createDefaultFont(this, "helvetica", 36);
				RiText.setDefaultColor(255);
				RiText.setBoundingBoxStrokeWeights(0);
				RiText.setBoundingBoxesVisible(true);

				color = 4;
				while (color == 4) 
					color = (int) (Math.random() * 4);

				RiText.setDefaultBBoxFill(colors[color]);

				RiText thisTitle;

				thisTitle = new RiText(this, titleArray[i], (int)(640*Math.random()), (int)(1000*Math.random()));
				// thisTitle.textFont( )
				thisTitle.setVisible(false);

				// add it to the array list.
				titleArrayList.add(thisTitle);
			}
		}

		titlesToDisplay.add(titleArrayList);
		// println("added");

	}

	/**
	 * Display RiText
	 */
	public void display(ArrayList<RiText> titleArrayList) {

		// println("display called");

		int start = 0;
		int i = 0;
		boolean canDisplay = true;
		int displayTimer = 10; // times the 'leftover' Google results once I've typed something

		// Start displaying them one by one
		while (i < (titleArrayList.size() + 9)) {

			if (millis() - start > attentionSpan){			

				if (isTyping == true) {
					canDisplay = false;
					displayTimer = (int) (10 - (attentionSpan / 1000) * 10);
					// println(displayTimer);
				}

				start = millis();

				if (i < titleArrayList.size() && (displayTimer > 0)) {

					titleArrayList.get(i).setVisible(true);

					if (!canDisplay)
						displayTimer--;
				}

				// Fade out, naturally
				if (i >= 9) {
					titleArrayList.get(i - 9).fadeOut(1f);	
				}

				if (i >= 10) {
					titleArrayList.get(i - 10).setVisible(false);
				}	

				i++;
			}
		}
	}

	/**
	 * Erase RiText
	 */
	public void erase() {

		if (!titlesToDisplay.isEmpty()) {

			int eraseTimer = millis();

			for (int i = titlesToDisplay.size() - 1; i >= 0; i--) {
				for (int j = titlesToDisplay.get(i).size() - 1; j >= 0; j--) {
					if (titlesToDisplay.get(i).get(j).isVisible()) {
						eraseTimer = millis();

						while (millis() - eraseTimer < (1100 - attentionSpan)) {

						}

						titlesToDisplay.get(i).get(j).fadeOut(1f);
					}
				}
			}
		}
	}

	/**
	 * Draws the layout and continually checks for lulls in my typing, then displaying
	 */
	public void draw() {

		background(255);

		// Create visual layout
		strokeWeight(0);

		// title bar
		stroke(96,152,200);
		fill(96,152,200);
		rect(0,0,width,60);

		// menu bar
		stroke(255);
		fill(0, 48, 96);
		rect(-1,60,width+1,20);

		// third bar
		stroke(96,152,200);
		fill(96,152,200);
		rect(0,81,width,20);

		// Livejournal logo
		image(logo,0,0);

		// entry box
		stroke(0);
		fill(255);
		rect(25, 225, 1000, 500);

		// Continually check for lull in typing, then displays

		if (millis() - lullStart > 200)
			isTyping = false;

		if (millis() - lullStart > attentionSpan && !titlesToDisplay.isEmpty()) {
			// println("display called, but can't call display");
			if (canCallDisplay) {

				// isTyping = false;

				Thread t = new Thread() {
					public void run() {
						canCallDisplay = false;
						cannotErase = false;
						display(titlesToDisplay.get(titlesToDisplay.size() - 1));
					}
				};
				t.start();

			}
		}
	}


    public static void main(String args[])
    {
      PApplet.main(new String[] { distraction.Distraction8.class.getName() });
    }

}

/**
 * BUGS
 * 
 * Occasionally it turns all to Helvetica. Occasionally the clouds all turn to Courier. 
 * 
 * DANIEL'S COMMENTS
 * 
 * Unsure whether it's a creativity enhancing thing, like Gideon's, or distracting
 * 
 * Exact replica of current LJ
 * Imitate aesthetics of Growl
 * Frame in the beginning - trying to get back to how I used LJ - "pretend it's 98" - why is that, was that 
 * important to me
 * and then first distraction - make that clear that it's a distraction
 * "Google widget" - people have to get what the setup, what the theatre is immediately. 
 * Maybe when I type about something - that would show that I'm responding to it - like highlight it. 
 * Human process that is automated - not sure how that ties
 * 
 * Could right click on a phrase - do it once or twice - "Always do this - automate" 
 * 
 * Livejournal API - sense of a finished piece
 * 
 * Make bold Helvetica. Create all your fonts in advance with Processing
 * Take the Google language. 
 * 
 * Processing - Tools - Create Font - data folder - 
 * .vlw 
 * Pfont helvetica = loadFont("helvetica.vlw");
 * When I have a RiText = rt.textFont(helvetica);
 * 
 * JEditorPane - html render - give it html, it renders - outputs it as graphics. 
 * 
 * Highlight - how do you 
 * createWords - change into lots of RiTexts. 
 * Takes string, a written array ---> outputs array of RiText, displays em all in nice lines
 * 		Example on Rednoise
 * 	bounding box, fonts, characters
 * 
 * *************************************************************
 * 
 * Imitate aesthetics of Growl - and Google
 * 		bold, if I have time
 * 
 * Unobtrusive Google search button - "search widget" 
 * New! "Search on Google"
 * Buttons: http://processing.org/learning/topics/buttons.html
 * 
 * After a few times - You have activated the Google search widget. 
 * THEN start the attentionSpan timer.
 *
 * When I write about it - highlight it? Maybe. 
 * 
 * Work on the setup of the writing. Stage the beginning, and the first distraction. Then let go.
 * 
 */